@@ -17,6 +17,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
 #include <hildon/hildon.h>
 #include <libhildondesktop/libhildondesktop.h>
@@ -127,17 +128,42 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   GtkWidget* icon_button;
   GtkWidget* icon_box;
   GtkWidget* display_name_box;
+  GtkWidget* connection_radio_box;
   GtkWidget* connection_name_box;
-  GtkWidget* connection_name_label;
+  GtkWidget* connection_option_box;
 
   // Connection Setting
-  connection_button = hildon_picker_button_new(HILDON_SIZE_THUMB_HEIGHT, 
+  connection_button = hildon_picker_button_new(HILDON_SIZE_FINGER_HEIGHT, 
 					       HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  hildon_button_set_title(HILDON_BUTTON(connection_button), "Connection");
 
   selector = hildon_touch_selector_new();
+
+  GtkWidget* on_connect_selector = hildon_touch_selector_new_text();
+  GtkWidget* on_disconnect_selector = hildon_touch_selector_new_text();
+
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(on_connect_selector),
+				    "-");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(on_connect_selector),
+				    "GSM");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(on_connect_selector),
+				    "3G");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(on_disconnect_selector),
+				    "-");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(on_disconnect_selector),
+				    "GSM");
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(on_disconnect_selector),
+				    "3G");
+
+  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(on_connect_selector),
+				   0, desktop_plugin->on_connect_action);
+  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(on_disconnect_selector),
+				   0, desktop_plugin->on_disconnect_action);
+
   connection_list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
   renderer = gtk_cell_renderer_text_new();
+
   column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(selector),
 					       GTK_TREE_MODEL(connection_list),
 					       renderer, "text", 0, NULL);
@@ -151,12 +177,30 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 
   init_connection_list(desktop_plugin, connection_list, selector, connection_button);
 
-  connection_name_box = gtk_hbox_new(TRUE, 3);
-  connection_name_label = gtk_label_new("Connection:");
+  
+  connection_option_box = gtk_hbox_new(TRUE, 0);
+  connection_name_box = gtk_vbox_new(TRUE, 0);
+  connection_radio_box = gtk_hbox_new(TRUE, 0);
 
-  gtk_box_pack_start(GTK_BOX(connection_name_box), connection_name_label, TRUE, TRUE, 3);
-  gtk_box_pack_end(GTK_BOX(connection_name_box), connection_button, TRUE, TRUE, 3);
+  GtkWidget* onConnectRadioModeChange = hildon_picker_button_new(HILDON_SIZE_FINGER_HEIGHT,
+								 HILDON_BUTTON_ARRANGEMENT_VERTICAL);
 
+  GtkWidget* onDisconnectRadioModeChange = hildon_picker_button_new(HILDON_SIZE_FINGER_HEIGHT,
+								    HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+
+  hildon_button_set_title(HILDON_BUTTON(onConnectRadioModeChange), "OnConnect");
+  hildon_button_set_title(HILDON_BUTTON(onDisconnectRadioModeChange), "OnDisconnect");
+  
+  hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(onConnectRadioModeChange),
+				    HILDON_TOUCH_SELECTOR(on_connect_selector));
+  hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(onDisconnectRadioModeChange),
+				    HILDON_TOUCH_SELECTOR(on_disconnect_selector));
+
+  gtk_box_pack_start(GTK_BOX(connection_radio_box), onConnectRadioModeChange, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(connection_radio_box), onDisconnectRadioModeChange, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(connection_name_box), connection_button, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(connection_option_box), connection_radio_box, FALSE, FALSE, 0);
+  gtk_box_pack_end(GTK_BOX(connection_option_box), connection_name_box, FALSE, FALSE, 0);
 
   // Display name
   display_name_entry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
@@ -174,9 +218,10 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 		   display_name_entry);
 
 
-  display_name_box = gtk_hbox_new(TRUE, 3);
-  gtk_box_pack_start(GTK_BOX(display_name_box), display_name_check_button, TRUE, TRUE, 3);
-  gtk_box_pack_end(GTK_BOX(display_name_box), display_name_entry, TRUE, TRUE, 3);
+  display_name_box = gtk_hbox_new(TRUE, 0);
+
+  gtk_box_pack_start(GTK_BOX(display_name_box), display_name_check_button, TRUE, TRUE, 0);
+  gtk_box_pack_end(GTK_BOX(display_name_box), display_name_entry, TRUE, TRUE, 0);
 
   g_signal_connect(G_OBJECT(connection_button),"value-changed",
 		   G_CALLBACK(connect_now_settings_connection_changed),
@@ -190,8 +235,8 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 
   icon_data_t* icon_data = connect_now_create_icon_data(desktop_plugin);
 
-  icon_box = gtk_hbox_new(FALSE, 3);
-  GtkWidget* icon_box2 = gtk_hbox_new(TRUE, 3);
+  icon_box = gtk_hbox_new(FALSE, 0);
+  GtkWidget* icon_box2 = gtk_hbox_new(TRUE, 0);
   gtk_box_pack_start(GTK_BOX(icon_box2), icon_data->connect_icon, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(icon_box2), icon_data->disconnect_icon, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(icon_box), icon_box2, TRUE, FALSE, 0);
@@ -215,7 +260,7 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   
   gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), icon_box, FALSE, FALSE, 3);
   gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), display_name_box, FALSE, FALSE, 3);
-  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), connection_name_box, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), connection_option_box, FALSE, FALSE, 3);
   gtk_box_pack_end(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), icon_size_scale, FALSE, FALSE, 3);
 
   gtk_widget_show_all(dialog);
@@ -253,6 +298,11 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
     GtkTreeIter iter;
     gboolean selected = hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(selector),
 							   0, &iter);
+    desktop_plugin->on_connect_action = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(on_connect_selector),
+									 0);
+    desktop_plugin->on_disconnect_action = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(on_disconnect_selector),
+									    0);
+    
     if(selected)
     {
       gchar* iap;
@@ -290,7 +340,144 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   g_object_unref(connection_list);
 }
 
-gboolean connect_now_connection_changed(ConIcConnection *connection, ConIcConnectionEvent *event, ConnectNowHomePlugin *desktop_plugin)
+void
+connect_now_change_radio_mode(ConnectNowHomePlugin* desktop_plugin, guint mode)
+{
+  DBusGConnection* dbus_conn = hd_home_plugin_item_get_dbus_g_connection(HD_HOME_PLUGIN_ITEM(desktop_plugin),
+									 DBUS_BUS_SYSTEM,
+									 NULL);
+  DBusGProxy *proxy = dbus_g_proxy_new_for_name(dbus_conn,
+						"com.nokia.phone.net",
+						"/com/nokia/phone/net",
+						"Phone.Net");
+  dbus_g_proxy_call_no_reply(proxy, 
+			     "set_selected_radio_access_technology",
+			     G_TYPE_UCHAR, 
+			     mode, 
+			     G_TYPE_INVALID);
+  g_object_unref(proxy);
+}
+
+guint 
+connect_now_get_current_radio_mode(ConnectNowHomePlugin* desktop_plugin)
+{
+  DBusGConnection* dbus_conn = hd_home_plugin_item_get_dbus_g_connection(HD_HOME_PLUGIN_ITEM(desktop_plugin),
+									 DBUS_BUS_SYSTEM,
+									 NULL);
+  DBusGProxy *proxy = dbus_g_proxy_new_for_name(dbus_conn,
+						"com.nokia.phone.net",
+						"/com/nokia/phone/net",
+						"Phone.Net");
+
+
+  guchar current_mode = 0;
+  gint dummy;
+  dbus_g_proxy_call(proxy, 
+		    "get_selected_radio_access_technology",
+		    NULL, G_TYPE_INVALID, 
+		    G_TYPE_UCHAR, 
+		    &current_mode, 
+		    G_TYPE_INT, 
+		    &dummy,
+		    G_TYPE_INVALID);
+  g_object_unref(proxy);
+  return current_mode;
+}
+
+static void
+handle_modem_state_changed(DBusGProxy* object, const gchar* mode, ConnectNowHomePlugin* desktop_plugin)
+{
+  if(g_strcmp0(mode, "online")==0) 
+  {
+    desktop_plugin->connect_now = TRUE;
+    dbus_g_proxy_disconnect_signal(object, 
+				   "modem_state_changed_ind", 
+				   G_CALLBACK(handle_modem_state_changed), 
+				   desktop_plugin); 
+    g_object_unref(object);
+  } 
+}
+
+static void
+connect_now_init_modem_state_listener(ConnectNowHomePlugin* desktop_plugin)
+{
+  DBusGConnection* dbus_conn = hd_home_plugin_item_get_dbus_g_connection(HD_HOME_PLUGIN_ITEM(desktop_plugin),
+									 DBUS_BUS_SYSTEM,
+									 NULL);
+  DBusGProxy *proxy = dbus_g_proxy_new_for_name(dbus_conn,
+						"com.nokia.phone.SSC",
+						"/com/nokia/phone/SSC",
+						"com.nokia.phone.SSC");
+
+  dbus_g_proxy_add_signal(proxy, 
+			  "modem_state_changed_ind", 
+			  G_TYPE_STRING, 
+			  G_TYPE_INVALID); 
+  dbus_g_proxy_connect_signal(proxy,
+			      "modem_state_changed_ind",
+			      G_CALLBACK(handle_modem_state_changed),
+			      desktop_plugin, NULL);
+}
+
+static void
+handle_operator_name_changed(DBusGProxy* object, guchar b, const gchar* name, const gchar* x, guint l, guint n, ConnectNowHomePlugin* desktop_plugin)
+{
+  if(strlen(name)!=0 && desktop_plugin->connect_now == TRUE)
+  {
+    con_ic_connection_connect_by_id(desktop_plugin->con, 
+				    desktop_plugin->connection_id,
+				    CON_IC_CONNECT_FLAG_NONE);
+    dbus_g_proxy_disconnect_signal(object,
+				   "operator_name_change",
+				   G_CALLBACK(handle_operator_name_changed),
+				   desktop_plugin);
+    g_object_unref(object);
+  }
+}
+
+static void
+connect_now_init_operator_name_change_listener(ConnectNowHomePlugin* desktop_plugin)
+{
+  DBusGConnection* dbus_conn = hd_home_plugin_item_get_dbus_g_connection(HD_HOME_PLUGIN_ITEM(desktop_plugin),
+									 DBUS_BUS_SYSTEM,
+									 NULL);
+  DBusGProxy *proxy = dbus_g_proxy_new_for_name(dbus_conn,
+						"com.nokia.phone.net",
+						"/com/nokia/phone/net",
+						"Phone.Net");
+  dbus_g_proxy_add_signal(proxy, 
+			  "operator_name_change", 
+			  G_TYPE_UCHAR, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT,
+			  G_TYPE_INVALID); 
+  dbus_g_proxy_connect_signal(proxy,
+			      "operator_name_change",
+			      G_CALLBACK(handle_operator_name_changed),
+			      desktop_plugin, NULL);
+}
+
+static void
+connect_now_switch_radio_mode_and_connect(ConnectNowHomePlugin* desktop_plugin)
+{
+  guint current_mode = connect_now_get_current_radio_mode(desktop_plugin);
+
+  if(current_mode!=desktop_plugin->on_connect_action)
+  {
+    desktop_plugin->connect_now = FALSE;
+    connect_now_init_modem_state_listener(desktop_plugin);
+    connect_now_init_operator_name_change_listener(desktop_plugin);
+    connect_now_change_radio_mode(desktop_plugin, desktop_plugin->on_connect_action);
+  }  
+  else
+  {
+    con_ic_connection_connect_by_id(desktop_plugin->con,  
+				    desktop_plugin->connection_id, 
+				    CON_IC_CONNECT_FLAG_NONE); 
+    desktop_plugin->connect_now = FALSE;
+  }
+}
+
+gboolean 
+connect_now_connection_changed(ConIcConnection *connection, ConIcConnectionEvent *event, ConnectNowHomePlugin *desktop_plugin)
 {
   const gchar* iap_id = con_ic_event_get_iap_id((ConIcEvent*) event);
 
@@ -330,11 +517,17 @@ gboolean connect_now_connection_changed(ConIcConnection *connection, ConIcConnec
     desktop_plugin->current_connection_id = NULL;
     if(desktop_plugin->connect_now)
     {
-      con_ic_connection_connect_by_id(desktop_plugin->con, 
-				      desktop_plugin->connection_id,
-				      CON_IC_CONNECT_FLAG_NONE);
-      
-      desktop_plugin->connect_now = FALSE;
+      if(desktop_plugin->on_connect_action != NO_CHANGE)
+      {
+	connect_now_switch_radio_mode_and_connect(desktop_plugin);
+      }
+      else
+      {
+	con_ic_connection_connect_by_id(desktop_plugin->con, 
+					desktop_plugin->connection_id,
+					CON_IC_CONNECT_FLAG_NONE);
+	desktop_plugin->connect_now = FALSE;
+      }
     }
   }
   return TRUE;
@@ -381,12 +574,12 @@ connect_now_switch_normal_mode(ConnectNowHomePlugin* desktop_plugin)
 
   gchar* dev_mode = NULL;
   dbus_g_proxy_call(proxy,
-					MCE_DEVICE_MODE_GET,
-					NULL,
-					G_TYPE_INVALID,
-					G_TYPE_STRING,
-					&dev_mode,
-					G_TYPE_INVALID);
+		    MCE_DEVICE_MODE_GET,
+		    NULL,
+		    G_TYPE_INVALID,
+		    G_TYPE_STRING,
+		    &dev_mode,
+		    G_TYPE_INVALID);
 
   if(g_str_equal(dev_mode, MCE_FLIGHT_MODE))
    dbus_g_proxy_call_no_reply(proxy,
@@ -411,7 +604,6 @@ connect_now_button_release(GtkWidget* widget, GdkEventButton *event, ConnectNowH
     desktop_plugin->connect_now = FALSE;
     return;
   }
-
   if(desktop_plugin->current_connection_id != NULL)
   {
     if(g_strcmp0(desktop_plugin->current_connection_id,
@@ -428,14 +620,31 @@ connect_now_button_release(GtkWidget* widget, GdkEventButton *event, ConnectNowH
       }
     }
     connect_now_disconnect_current_connection(desktop_plugin);
+    if(desktop_plugin->connect_now == FALSE &&
+       desktop_plugin->on_disconnect_action != NO_CHANGE)
+    {
+      guint current_mode = connect_now_get_current_radio_mode(desktop_plugin);
+      if(current_mode!=desktop_plugin->on_disconnect_action)
+      {
+	connect_now_change_radio_mode(desktop_plugin, desktop_plugin->on_disconnect_action);
+      }
+    }
   }
   else
   {
     connect_now_switch_normal_mode(desktop_plugin);
-    con_ic_connection_connect_by_id(desktop_plugin->con, 
-				    desktop_plugin->connection_id,
-				    CON_IC_CONNECT_FLAG_NONE);
-    desktop_plugin->connect_now = FALSE;
+    if(desktop_plugin->on_connect_action != NO_CHANGE)
+    {
+      desktop_plugin->connect_now = TRUE;
+      connect_now_switch_radio_mode_and_connect(desktop_plugin);
+    }
+    else
+    {
+      con_ic_connection_connect_by_id(desktop_plugin->con,  
+				      desktop_plugin->connection_id, 
+				      CON_IC_CONNECT_FLAG_NONE); 
+      desktop_plugin->connect_now = FALSE;
+    }
   }
   gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
   return FALSE;
@@ -455,6 +664,8 @@ connect_now_home_plugin_init(ConnectNowHomePlugin *desktop_plugin)
   desktop_plugin->con = con_ic_connection_new();
   desktop_plugin->widget_size = CONNECTNOW_DEFAULT_WIDGET_WIDTH;
   desktop_plugin->hide_connection_name = TRUE;
+  desktop_plugin->on_connect_action = NO_CHANGE;
+  desktop_plugin->on_disconnect_action = NO_CHANGE;
   g_signal_connect(desktop_plugin->con, "connection-event", G_CALLBACK(connect_now_connection_changed), desktop_plugin);
   g_object_set(desktop_plugin->con, "automatic-connection-events", TRUE, NULL);
 
