@@ -71,7 +71,8 @@ struct _ConnectNowHomePlugin
 
   GdkPixbuf* fallback_connectedImage;
   GdkPixbuf* fallback_disconnectedImage;
-
+  guint on_connect_action;
+  guint on_disconnect_action;
 };
 
 struct _ConnectNowHomePluginClass
