@@ -23,6 +23,11 @@
 #include "connectnow-home-widget.h"
 
 static const int CONNECTNOW_DEFAULT_WIDGET_WIDTH = 96;
+enum CONNECT_CHANGE_ACTION {
+  NO_CHANGE = 0,
+  CHANGE_TO_GSM,
+  CHANGE_TO_3G
+};
 
 void 
 read_settings(ConnectNowHomePlugin *desktop_plugin);
